"use strict";

/* exported logger */

var Article = require("./Article");

var logger = require('../../logger');

/**
 * @typedef {Object} AnnotatorOptions
 * @property {string} className
 * @property {string[]} elementClassNames
 * @property {string} name
 * @property {string} description
 * @property {AnnotatorParameterDefinition[]} parametersDefinitions
 * @property {string} url
 */

/**
 * @typedef {Object} AnnotatorParameterDefinition
 * @property {string} description
 * @property {string} name
 * @property {string} type
 */

/**
 *
 * @param {Annotator|AnnotatorOptions} javaObject
 * @param {Configuration} configuration
 * @constructor
 */
function Annotator(javaObject, configuration) {
  var self = this;
  if (javaObject instanceof Annotator) {
    self.setClassName(javaObject.getClassName());
    this._elementTypes = javaObject.getElementTypes();
    self.setName(javaObject.getName());
    self.setDescription(javaObject.getDescription());
    self.setParametersDefinitions(javaObject.getParametersDefinitions());
    self.setUrl(javaObject.getUrl());
  } else {
    self.setClassName(javaObject.className);
    self.setElementTypes(javaObject.elementClassNames, configuration);
    self.setName(javaObject.name);
    self.setDescription(javaObject.description);
    self.setParametersDefinitions(javaObject.parametersDefinitions);
    self.setUrl(javaObject.url);
  }
}

/**
 *
 * @param {string} className
 */
Annotator.prototype.setClassName = function (className) {
  this._className = className;
};

/**
 *
 * @returns {string}
 */
Annotator.prototype.getClassName = function () {
  return this._className;
};

/**
 *
 * @param {string} name
 */
Annotator.prototype.setName = function (name) {
  this._name = name;
};

/**
 *
 * @returns {string}
 */
Annotator.prototype.getName = function () {
  return this._name;
};

/**
 *
 * @param {string} description
 */
Annotator.prototype.setDescription = function (description) {
  this._description = description;
};

/**
 *
 * @returns {string}
 */
Annotator.prototype.getDescription = function () {
  return this._description;
};

/**
 *
 * @param {string} url
 */
Annotator.prototype.setUrl = function (url) {
  this._url = url;
};

/**
 *
 * @returns {string}
 */
Annotator.prototype.getUrl = function () {
  return this._url;
};

/**
 *
 * @returns {AnnotatorParameterDefinition[]}
 */
Annotator.prototype.getParametersDefinitions = function () {
  return this._parametersDefinitions;
};

/**
 *
 * @param {AnnotatorParameterDefinition[]} parametersDefinitions
 */
Annotator.prototype.setParametersDefinitions = function (parametersDefinitions) {
  this._parametersDefinitions = parametersDefinitions;
};

/**
 *
 * @param {string[]} elementTypesClassNames
 * @param {Configuration} configuration
 */
Annotator.prototype.setElementTypes = function (elementTypesClassNames, configuration) {
  this._elementTypes = [];
  var typeByClassName = {};
  var types = configuration.getElementTypes();
  var i, type;
  for (i = 0; i < types.length; i++) {
    type = types[i];
    typeByClassName[type.className] = type;
  }
  types = configuration.getReactionTypes();
  for (i = 0; i < types.length; i++) {
    type = types[i];
    typeByClassName[type.className] = type;
  }

  for (i = 0; i < elementTypesClassNames.length; i++) {
    type = typeByClassName[elementTypesClassNames[i]];
    if (type !== undefined) {
      this._elementTypes.push(type)
    } else {
      throw new Error("Unknown elementType: " + elementTypesClassNames[i]);
    }
  }
};

/**
 *
 * @returns {BioEntityType[]}
 */
Annotator.prototype.getElementTypes = function () {
  return this._elementTypes;
};


module.exports = Annotator;
